/*
 *	Copyright (C) 2008 by Chebotarev Roman <roma@ultranet.ru>
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#define RELEASE_DATE        "13.08.2008"
#define PROG_NAME			"DHCP-dropper"
#define BASE_VERSION        PROG_NAME" v0.3.2"
#define VERSION             BASE_VERSION

#define COPYRIGHT           VERSION" was written by Chebotarev Roman at "RELEASE_DATE"\n"

#define DEF_TTL             64
#define IP_HDR_LEN          5       /*IP header len in 32-bit words*/
#define B_IP_HDR_LEN        5 * 4   /*IP header len in bytes*/
#define DEF_SERVER_PORT     67
#define DEF_CLIENT_PORT     68
#define MAX_PORT_VALUE      65535

#define DHCP_UDP_OVERHEAD   (20 +   /* IP header */           \
                 8)                 /* UDP header */
#define DHCP_SNAME_LEN      64
#define DHCP_FILE_LEN       128
#define DHCP_FIXED_NON_UDP  236
#define DHCP_FIXED_LEN      (DHCP_FIXED_NON_UDP + DHCP_UDP_OVERHEAD)
                        /* Everything but options. */
#define DHCP_MTU_MAX        1500
#define DHCP_OPTION_LEN     (DHCP_MTU_MAX - DHCP_FIXED_LEN)

#define BOOTP_MIN_LEN       300
#define DHCP_MIN_LEN        548

#define MAX_ADDR_COUNT      255
#define MAX_ATTEMPTS        5
#define DEF_HOSTNAME        "DHCP-dropper"
#define DEF_DHCPCLIENT      DEF_HOSTNAME
#define STR_DHCP_FILTER     "(ether dst host %s or ether dst host FF:FF:FF:FF:FF:FF) and udp and src port %d and dst port %d"

/*Defined errors codes*/
#define ERR_EUID                10
#define ERR_SIGNAL              20
#define ERR_CONFIG              30
#define ERR_MEMORY              40
#define ERR_OPENDEV             50
#define ERR_FILTER_OWERFLOW     60
#define ERR_PCAP_COMPILE        70
#define ERR_PCAP_SETFILTER      80
#define ERR_SENDPACKET          90
#define ERR_GETPACKET           100
#define ERR_SETNOBLOCK          110
#define ILLEGAL_SERV_FOUND      200

#ifdef  __FreeBSD__
#define PCAP_OPEN_LIVE_TIMEOUT	1
#elif	__linux__
#define	PCAP_OPEN_LIVE_TIMEOUT	-1
#else 
#error "Unsupported system! Supported is Linux & FreeBSD."
#endif

/* UDP header as specified by RFC 768, August 1980. */

struct udphdr
{
	u_int16_t source;
	u_int16_t dest;
	u_int16_t len;
	u_int16_t check;
};

/*
 * Structure of an internet header, naked of options.
 */
struct iphdr
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
    unsigned int ihl:4;
    unsigned int version:4;
#elif __BYTE_ORDER == __BIG_ENDIAN
    unsigned int version:4;
    unsigned int ihl:4;
#else
# error "Please fix <bits/endian.h>"
#endif
    u_int8_t tos;
    u_int16_t tot_len;
    u_int16_t id;
    u_int16_t frag_off;
    u_int8_t ttl;
    u_int8_t protocol;
    u_int16_t check;
    u_int32_t saddr;
    u_int32_t daddr;
    /*The options start here. */
};

const uint8_t magic_cookie[]={99, 130, 83, 99};

struct ignored_mac_node
{
    char mac_addr[19];
    struct ignored_mac_node * next;
};

#pragma pack(1)

struct pseudo_header
{
    unsigned int src_addr; 
    unsigned int dst_addr;
    unsigned char zero ; 
    unsigned char proto;
    unsigned short length;
}; 

struct full_packet_header
{
    struct ether_header eth_head;
    struct iphdr ip_header;
    struct udphdr udp_header;
};

struct config_params
{
    char    test_mode;
    char    yes;
    char    rand_mac_always;
    int     max_attempts;
    char    *if_name;
    int     max_address_count;
    char    client_hostname[64];
    char    dhcp_client[16];
    char    str_source_mac[19];
    char    flood;
    int     children_count;
    char    broadcast;
    int     client_port;
    int     server_port;
    char    always_wait_from_67;
    char    always_wait_to_68;
};

